#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef JFB_ENABLE_SPLASH_SCREEN
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "read-bmp.h"
#include "message.h"

u_char *
read_bmp_file(FILE *fp, int *width, int *height)
{
	struct bmp_file *bmp;
	u_int data_size;
	int x, y, line_len;
	int tbuf;
	u_char *data;

	if((bmp = (struct bmp_file *)calloc(1, sizeof(struct bmp_file))) == NULL) {
		print_message("(UXP Splash) read-bmp: ERROR: malloc error (struct bmp_file)\n");
		return NULL;
	}

	if((fgetc(fp) | (fgetc(fp) << 8)) != BMP_SIGNATURE) {
		print_message("(UXP Splash) read-bmp: ERROR: file is not a BMP file\n");
		free(bmp);
		return NULL;
	}

	fread(&(bmp->hdr), sizeof(struct bmp_hdr), 1, fp);

	if(bmp->hdr.compression != 0) {
		print_message("(UXP Splash) read-bmp: ERROR: we can only handle uncompressed BMP files\n");
		free(bmp);
		return NULL;
	}

	*width = bmp->hdr.width;
	*height = bmp->hdr.height;
	line_len = *width * 3;

	/* 32 bit boundary for each line */
	if(line_len % 4) {
		line_len = line_len + (4 - (line_len % 4));
	}

	if(bmp->hdr.bit_count == 24) {
		data_size = line_len * *height;
	}
	else {
		print_message("(UXP Splash) read-bmp: ERROR: we only support 24 bit BMP files\n");
		free(bmp);
		return NULL;
	}

	fseek(fp, bmp->hdr.offset, SEEK_SET);
	if((bmp->data = (u_char *)malloc(data_size * sizeof(u_char))) == NULL) {
		fprintf(stderr, "Error: read_bmp_file, malloc error (bmp data)\n");
		free(bmp);
		return NULL;
	}

	fread(bmp->data, 1, data_size, fp);

	if((data = (u_char *)malloc((*width * 3) * *height * sizeof(u_char))) == NULL) {
		fprintf(stderr, "Error: read_bmp_file, malloc error\n");
		free(bmp->data);
		free(bmp);
		return NULL;
	}

	if(bmp->hdr.bit_count == 24) {
		for(y = 0; y < *height; y++) {
			memcpy(data + y * (*width * 3), bmp->data + (*height - y - 1) * line_len, (*width * 3));
		}
		for(y = 0; y < *height; y++) {
			for(x = 0; x < (*width * 3); x = x + 3) {
				tbuf = data[y * (*width * 3) + x];
				data[y * (*width * 3) + x] = data[y * (*width * 3) + x + 2];
				data[y * (*width * 3) + x + 2] = tbuf;
			}
		}

	}

	free(bmp->data);
	free(bmp);

	return data;
}
#endif
